@@ -450,6 +450,9 @@ _SLang_Token_Type;
 extern void _SLcompile (_SLang_Token_Type *);
 extern void (*_SLcompile_ptr)(_SLang_Token_Type *);
 
+/* slmisc.c */
+extern char *_SLskip_whitespace (char *s);
+
 /* *** TOKENS *** */
 
 /* Note that that tokens corresponding to ^J, ^M, and ^Z should not be used.
