@@ -1,14 +1,14 @@
 #ifndef DAVIS_SLANG_H_
 #define DAVIS_SLANG_H_
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
-#define SLANG_VERSION 10403
-#define SLANG_VERSION_STRING "1.4.3"
+#define SLANG_VERSION 10406
+#define SLANG_VERSION_STRING "1.4.6"
 
 /*{{{ System Dependent Macros and Typedefs */
 
@@ -67,7 +67,7 @@
 # endif
 #endif
 
-#if defined(__NT__) || defined(__MINGW32__) || defined(__CYGWIN32__)
+#if defined(__NT__) || defined(__MINGW32__) /* || defined(__CYGWIN32__) */
 # ifndef IBMPC_SYSTEM
 #  define IBMPC_SYSTEM
 # endif
@@ -113,7 +113,7 @@ extern "C" {
 
 typedef int (*FVOID_STAR)(void);
 
-#if defined(__MSDOS_) && defined(__BORLANDC__)
+#if defined(__MSDOS__) && defined(__BORLANDC__)
 # define SLFREE(buf)  farfree((void far *)(buf))
 # define SLMALLOC(x) farmalloc((unsigned long) (x))
 # define SLREALLOC(buf, n) farrealloc((void far *) (buf), (unsigned long) (n))
@@ -150,6 +150,8 @@ typedef int (*FVOID_STAR)(void);
 
 /*{{{ Interpreter Typedefs */
 
+typedef unsigned char SLtype;	       /* This will be unsigned int in V2 */
+
 typedef struct _SLang_Name_Type
 {
    char *name;
@@ -187,9 +189,9 @@ typedef struct
 
    /* Do not change this without modifying slang.c:execute_intrinsic_fun */
 #define SLANG_MAX_INTRIN_ARGS	7
-   unsigned char arg_types [SLANG_MAX_INTRIN_ARGS];
+   SLtype arg_types [SLANG_MAX_INTRIN_ARGS];
    unsigned char num_args;
-   unsigned char return_type;
+   SLtype return_type;
 }
 SLang_Intrin_Fun_Type;
 
@@ -200,7 +202,7 @@ typedef struct
    char name_type;
 
    VOID_STAR addr;
-   unsigned char type;
+   SLtype type;
 }
 SLang_Intrin_Var_Type;
 
@@ -246,7 +248,7 @@ typedef struct
 {
    char *field_name;
    unsigned int offset;
-   unsigned char type;
+   SLtype type;
    unsigned char read_only;
 }
 SLang_IStruct_Field_Type;
@@ -272,6 +274,10 @@ extern int SLns_add_istruct_table (SLang_NameSpace_Type *, SLang_IStruct_Field_T
 extern SLang_NameSpace_Type *SLns_create_namespace (char *);
 extern void SLns_delete_namespace (SLang_NameSpace_Type *);
 
+extern int SLns_load_file (char *, char *);
+extern int SLns_load_string (char *, char *);
+extern int (*SLns_Load_File_Hook) (char *, char *);
+
 typedef struct SLang_Load_Type
 {
    int type;
@@ -297,13 +303,15 @@ typedef struct SLang_Load_Type
     * the compilable unit.
     */
 
-   unsigned long reserved[4];
+   char *namespace_name;
+   unsigned long reserved[3];
    /* For future expansion */
 } SLang_Load_Type;
 
 extern SLang_Load_Type *SLallocate_load_type (char *);
 extern void SLdeallocate_load_type (SLang_Load_Type *);
-
+extern SLang_Load_Type *SLns_allocate_load_type (char *, char *);
+  
 /* Returns SLang_Error upon failure */
 extern int SLang_load_object (SLang_Load_Type *);
 extern int (*SLang_Load_File_Hook)(char *);
@@ -326,37 +334,40 @@ extern int SLang_generate_debug_info (int);
 
 typedef struct SL_OOBinary_Type
 {
-   unsigned char data_type;	       /* partner type for binary op */
+   SLtype data_type;	       /* partner type for binary op */
 
     int (*binary_function)_PROTO((int,
-				 unsigned char, VOID_STAR, unsigned int,
-				 unsigned char, VOID_STAR, unsigned int,
+				 SLtype, VOID_STAR, unsigned int,
+				 SLtype, VOID_STAR, unsigned int,
 				 VOID_STAR));
 
-   int (*binary_result) _PROTO((int, unsigned char, unsigned char, unsigned char *));
+   int (*binary_result) _PROTO((int, SLtype, SLtype, SLtype *));
    struct SL_OOBinary_Type *next;
 }
 SL_OOBinary_Type;
 
 typedef struct _SL_Typecast_Type
 {
-   unsigned char data_type;	       /* to_type */
+   SLtype data_type;	       /* to_type */
    int allow_implicit;
 
-   int (*typecast)_PROTO((unsigned char, VOID_STAR, unsigned int,
-			  unsigned char, VOID_STAR));
+   int (*typecast)_PROTO((SLtype, VOID_STAR, unsigned int,
+			  SLtype, VOID_STAR));
    struct _SL_Typecast_Type *next;
 }
 SL_Typecast_Type;
 
 typedef struct _SLang_Struct_Type SLang_Struct_Type;
-
-#if defined(SL_APP_WANTS_FOREACH)
-/* It is up to the application to define struct _SLang_Foreach_Context_Type */
 typedef struct _SLang_Foreach_Context_Type SLang_Foreach_Context_Type;
+
+#if 0
+#if defined(SL_APP_WANTS_FOREACH)
+typedef struct _SLang_Foreach_Context_Type SLang_Foreach_Context_Type;
+/* It is up to the application to define struct _SLang_Foreach_Context_Type */
 #else
 typedef int SLang_Foreach_Context_Type;
 #endif
+#endif
 
 typedef struct
 {
@@ -375,7 +386,7 @@ typedef struct
    /* Methods */
 
    /* Most of the method functions are prototyped:
-    * int method (unsigned char type, VOID_STAR addr);
+    * int method (SLtype type, VOID_STAR addr);
     * Here, @type@ represents the type of object that the method is asked
     * to deal with.  The second parameter @addr@ will contain the ADDRESS of
     * the object.  For example, if type is SLANG_INT_TYPE, then @addr@ will
@@ -383,32 +394,32 @@ typedef struct
     * then @addr@ will contain the address of the string, i.e., char **.
     */
 
-   void (*cl_destroy)_PROTO((unsigned char, VOID_STAR));
+   void (*cl_destroy)_PROTO((SLtype, VOID_STAR));
    /* Prototype: void destroy(unsigned type, VOID_STAR val)
     * Called to delete/free the object */
 
-   char *(*cl_string)_PROTO((unsigned char, VOID_STAR));
-   /* Prototype: char *to_string (unsigned char t, VOID_STAR p);
+   char *(*cl_string)_PROTO((SLtype, VOID_STAR));
+   /* Prototype: char *to_string (SLtype t, VOID_STAR p);
     * Here p is a pointer to the object for which a string representation
     * is to be returned.  The returned pointer is to be a MALLOCED string.
     */
 
-   /* Prototype: void push(unsigned char type, VOID_STAR v);
+   /* Prototype: void push(SLtype type, VOID_STAR v);
     * Push a copy of the object of type @type@ at address @v@ onto the
     * stack.
     */
-   int (*cl_push)_PROTO((unsigned char, VOID_STAR));
+   int (*cl_push)_PROTO((SLtype, VOID_STAR));
 
-   /* Prototype: int pop(unsigned char type, VOID_STAR v);
+   /* Prototype: int pop(SLtype type, VOID_STAR v);
     * Pops value from stack and assign it to object, whose address is @v@.
     */
-   int (*cl_pop)_PROTO((unsigned char, VOID_STAR));
+   int (*cl_pop)_PROTO((SLtype, VOID_STAR));
 
-   int (*cl_unary_op_result_type)_PROTO((int, unsigned char, unsigned char *));
-   int (*cl_unary_op)_PROTO((int, unsigned char, VOID_STAR, unsigned int, VOID_STAR));
+   int (*cl_unary_op_result_type)_PROTO((int, SLtype, SLtype *));
+   int (*cl_unary_op)_PROTO((int, SLtype, VOID_STAR, unsigned int, VOID_STAR));
 
-   int (*cl_app_unary_op_result_type)_PROTO((int, unsigned char, unsigned char *));
-   int (*cl_app_unary_op)_PROTO((int, unsigned char, VOID_STAR, unsigned int, VOID_STAR));
+   int (*cl_app_unary_op_result_type)_PROTO((int, SLtype, SLtype *));
+   int (*cl_app_unary_op)_PROTO((int, SLtype, VOID_STAR, unsigned int, VOID_STAR));
 
    /* If this function is non-NULL, it will be called for sin, cos, etc... */
 #define SLMATH_SIN	1
@@ -432,133 +443,135 @@ typedef struct
 #define SLMATH_TODOUBLE	19
 #define SLMATH_CONJ	20
 
-   int (*cl_math_op)_PROTO((int, unsigned char, VOID_STAR, unsigned int, VOID_STAR));
-   int (*cl_math_op_result_type)_PROTO((int, unsigned char, unsigned char *));
+   int (*cl_math_op)_PROTO((int, SLtype, VOID_STAR, unsigned int, VOID_STAR));
+   int (*cl_math_op_result_type)_PROTO((int, SLtype, SLtype *));
 
    SL_OOBinary_Type *cl_binary_ops;
    SL_Typecast_Type *cl_typecast_funs;
 
-   void (*cl_byte_code_destroy)_PROTO((unsigned char, VOID_STAR));
-   void (*cl_user_destroy_fun)_PROTO((unsigned char, VOID_STAR));
-   int (*cl_init_array_object)_PROTO((unsigned char, VOID_STAR));
-   int (*cl_datatype_deref)_PROTO((unsigned char));
+   void (*cl_byte_code_destroy)_PROTO((SLtype, VOID_STAR));
+   void (*cl_user_destroy_fun)_PROTO((SLtype, VOID_STAR));
+   int (*cl_init_array_object)_PROTO((SLtype, VOID_STAR));
+   int (*cl_datatype_deref)_PROTO((SLtype));
    SLang_Struct_Type *cl_struct_def;
-   int (*cl_dereference) _PROTO((unsigned char, VOID_STAR));
-   int (*cl_acopy) (unsigned char, VOID_STAR, VOID_STAR);
-   int (*cl_apop) _PROTO((unsigned char, VOID_STAR));
-   int (*cl_apush) _PROTO((unsigned char, VOID_STAR));
-   int (*cl_push_literal) _PROTO((unsigned char, VOID_STAR));
-   void (*cl_adestroy)_PROTO((unsigned char, VOID_STAR));
-   int (*cl_push_intrinsic)_PROTO((unsigned char, VOID_STAR));
-   int (*cl_void_typecast)_PROTO((unsigned char, VOID_STAR, unsigned int, unsigned char, VOID_STAR));
+   int (*cl_dereference) _PROTO((SLtype, VOID_STAR));
+   int (*cl_acopy) (SLtype, VOID_STAR, VOID_STAR);
+   int (*cl_apop) _PROTO((SLtype, VOID_STAR));
+   int (*cl_apush) _PROTO((SLtype, VOID_STAR));
+   int (*cl_push_literal) _PROTO((SLtype, VOID_STAR));
+   void (*cl_adestroy)_PROTO((SLtype, VOID_STAR));
+   int (*cl_push_intrinsic)_PROTO((SLtype, VOID_STAR));
+   int (*cl_void_typecast)_PROTO((SLtype, VOID_STAR, unsigned int, SLtype, VOID_STAR));
 
-   int (*cl_anytype_typecast)_PROTO((unsigned char, VOID_STAR, unsigned int, unsigned char, VOID_STAR));
+   int (*cl_anytype_typecast)_PROTO((SLtype, VOID_STAR, unsigned int, SLtype, VOID_STAR));
 
    /* Array access functions */
-   int (*cl_aput) (unsigned char, unsigned int);
-   int (*cl_aget) (unsigned char, unsigned int);
-   int (*cl_anew) (unsigned char, unsigned int);
+   int (*cl_aput) (SLtype, unsigned int);
+   int (*cl_aget) (SLtype, unsigned int);
+   int (*cl_anew) (SLtype, unsigned int);
 
    /* length method */
-   int (*cl_length) (unsigned char, VOID_STAR, unsigned int *);
+   int (*cl_length) (SLtype, VOID_STAR, unsigned int *);
 
    /* foreach */
-   SLang_Foreach_Context_Type *(*cl_foreach_open) (unsigned char, unsigned int);
-   void (*cl_foreach_close) (unsigned char, SLang_Foreach_Context_Type *);
-   int (*cl_foreach) (unsigned char, SLang_Foreach_Context_Type *);
+   SLang_Foreach_Context_Type *(*cl_foreach_open) (SLtype, unsigned int);
+   void (*cl_foreach_close) (SLtype, SLang_Foreach_Context_Type *);
+   int (*cl_foreach) (SLtype, SLang_Foreach_Context_Type *);
 
    /* Structure access: get and put (assign to) fields */
-   int (*cl_sput) (unsigned char, char *);
-   int (*cl_sget) (unsigned char, char *);
+   int (*cl_sput) (SLtype, char *);
+   int (*cl_sget) (SLtype, char *);
 
    /* File I/O */
-   int (*cl_fread) (unsigned char, FILE *, VOID_STAR, unsigned int, unsigned int *);
-   int (*cl_fwrite) (unsigned char, FILE *, VOID_STAR, unsigned int, unsigned int *);
-   int (*cl_fdread) (unsigned char, int, VOID_STAR, unsigned int, unsigned int *);
-   int (*cl_fdwrite) (unsigned char, int, VOID_STAR, unsigned int, unsigned int *);
+   int (*cl_fread) (SLtype, FILE *, VOID_STAR, unsigned int, unsigned int *);
+   int (*cl_fwrite) (SLtype, FILE *, VOID_STAR, unsigned int, unsigned int *);
+   int (*cl_fdread) (SLtype, int, VOID_STAR, unsigned int, unsigned int *);
+   int (*cl_fdwrite) (SLtype, int, VOID_STAR, unsigned int, unsigned int *);
 
-   int (*cl_to_bool) (unsigned char, int *);
+   int (*cl_to_bool) (SLtype, int *);
    
-   int (*cl_cmp)(unsigned char, VOID_STAR, VOID_STAR, int *);
-
+   int (*cl_cmp)(SLtype, VOID_STAR, VOID_STAR, int *);   
 } SLang_Class_Type;
 
 /* These are the low-level functions for building push/pop methods.  They
  * know nothing about memory management.  For SLANG_CLASS_TYPE_MMT, use the
  * MMT push/pop functions instead.
  */
-extern int SLclass_push_double_obj (unsigned char, double);
-extern int SLclass_push_float_obj (unsigned char, float);
-extern int SLclass_push_long_obj (unsigned char, long);
-extern int SLclass_push_int_obj (unsigned char, int);
-extern int SLclass_push_short_obj (unsigned char, short);
-extern int SLclass_push_char_obj (unsigned char, char);
-extern int SLclass_push_ptr_obj (unsigned char, VOID_STAR);
-extern int SLclass_pop_double_obj (unsigned char, double *);
-extern int SLclass_pop_float_obj (unsigned char, float *);
-extern int SLclass_pop_long_obj (unsigned char, long *);
-extern int SLclass_pop_int_obj (unsigned char, int *);
-extern int SLclass_pop_short_obj (unsigned char, short *);
-extern int SLclass_pop_char_obj (unsigned char, char *);
-extern int SLclass_pop_ptr_obj (unsigned char, VOID_STAR *);
+extern int SLclass_push_double_obj (SLtype, double);
+extern int SLclass_push_float_obj (SLtype, float);
+extern int SLclass_push_long_obj (SLtype, long);
+extern int SLclass_push_int_obj (SLtype, int);
+extern int SLclass_push_short_obj (SLtype, short);
+extern int SLclass_push_char_obj (SLtype, char);
+extern int SLclass_push_ptr_obj (SLtype, VOID_STAR);
+extern int SLclass_pop_double_obj (SLtype, double *);
+extern int SLclass_pop_float_obj (SLtype, float *);
+extern int SLclass_pop_long_obj (SLtype, long *);
+extern int SLclass_pop_int_obj (SLtype, int *);
+extern int SLclass_pop_short_obj (SLtype, short *);
+extern int SLclass_pop_char_obj (SLtype, char *);
+extern int SLclass_pop_ptr_obj (SLtype, VOID_STAR *);
 
 extern SLang_Class_Type *SLclass_allocate_class (char *);
 extern int SLclass_get_class_id (SLang_Class_Type *cl);
-extern int SLclass_create_synonym (char *, unsigned char);
-extern int SLclass_is_class_defined (unsigned char);
+extern int SLclass_create_synonym (char *, SLtype);
+extern int SLclass_is_class_defined (SLtype);
+extern int SLclass_dup_object (SLtype type, VOID_STAR from, VOID_STAR to);
 
-extern int SLclass_register_class (SLang_Class_Type *, unsigned char, unsigned int, unsigned char);
-extern int SLclass_set_string_function (SLang_Class_Type *, char *(*)(unsigned char, VOID_STAR));
-extern int SLclass_set_destroy_function (SLang_Class_Type *, void (*)(unsigned char, VOID_STAR));
-extern int SLclass_set_push_function (SLang_Class_Type *, int (*)(unsigned char, VOID_STAR));
-extern int SLclass_set_pop_function (SLang_Class_Type *, int (*)(unsigned char, VOID_STAR));
+extern int SLclass_register_class (SLang_Class_Type *, SLtype, unsigned int, SLtype);
+extern int SLclass_set_string_function (SLang_Class_Type *, char *(*)(SLtype, VOID_STAR));
+extern int SLclass_set_destroy_function (SLang_Class_Type *, void (*)(SLtype, VOID_STAR));
+extern int SLclass_set_push_function (SLang_Class_Type *, int (*)(SLtype, VOID_STAR));
+extern int SLclass_set_pop_function (SLang_Class_Type *, int (*)(SLtype, VOID_STAR));
 
-extern int SLclass_set_aget_function (SLang_Class_Type *, int (*)(unsigned char, unsigned int));
-extern int SLclass_set_aput_function (SLang_Class_Type *, int (*)(unsigned char, unsigned int));
-extern int SLclass_set_anew_function (SLang_Class_Type *, int (*)(unsigned char, unsigned int));
+extern int SLclass_set_aget_function (SLang_Class_Type *, int (*)(SLtype, unsigned int));
+extern int SLclass_set_aput_function (SLang_Class_Type *, int (*)(SLtype, unsigned int));
+extern int SLclass_set_anew_function (SLang_Class_Type *, int (*)(SLtype, unsigned int));
 
-extern int SLclass_set_sget_function (SLang_Class_Type *, int (*)(unsigned char, char *));
-extern int SLclass_set_sput_function (SLang_Class_Type *, int (*)(unsigned char, char *));
+extern int SLclass_set_sget_function (SLang_Class_Type *, int (*)(SLtype, char *));
+extern int SLclass_set_sput_function (SLang_Class_Type *, int (*)(SLtype, char *));
+
+extern int SLclass_set_acopy_function (SLang_Class_Type *, int (*)(SLtype, VOID_STAR, VOID_STAR));
 
 /* Typecast object on the stack to type p1.  p2 and p3 should be set to 1 */
-extern int SLclass_typecast (unsigned char, int, int);
+extern int SLclass_typecast (SLtype, int, int);
 
-extern int SLclass_add_unary_op (unsigned char,
+extern int SLclass_add_unary_op (SLtype,
 				 int (*) (int,
-					  unsigned char, VOID_STAR, unsigned int,
+					  SLtype, VOID_STAR, unsigned int,
 					  VOID_STAR),
-				 int (*) (int, unsigned char, unsigned char *));
+				 int (*) (int, SLtype, SLtype *));
 
 extern int
-SLclass_add_app_unary_op (unsigned char,
+SLclass_add_app_unary_op (SLtype,
 			  int (*) (int,
-				   unsigned char, VOID_STAR, unsigned int,
+				   SLtype, VOID_STAR, unsigned int,
 				   VOID_STAR),
-			  int (*) (int, unsigned char, unsigned char *));
+			  int (*) (int, SLtype, SLtype *));
 
 extern int
-SLclass_add_binary_op (unsigned char, unsigned char,
+SLclass_add_binary_op (SLtype, SLtype,
 		       int (*) (int,
-				unsigned char, VOID_STAR, unsigned int,
-				unsigned char, VOID_STAR, unsigned int,
+				SLtype, VOID_STAR, unsigned int,
+				SLtype, VOID_STAR, unsigned int,
 				VOID_STAR),
-		       int (*) (int, unsigned char, unsigned char, unsigned char *));
+		       int (*) (int, SLtype, SLtype, SLtype *));
 
 extern int
-SLclass_add_math_op (unsigned char,
+SLclass_add_math_op (SLtype,
 		     int (*)(int,
-			     unsigned char, VOID_STAR, unsigned int,
+			     SLtype, VOID_STAR, unsigned int,
 			     VOID_STAR),
-		     int (*)(int, unsigned char, unsigned char *));
+		     int (*)(int, SLtype, SLtype *));
 
 extern int
-SLclass_add_typecast (unsigned char /* from */, unsigned char /* to */,
-		      int (*)_PROTO((unsigned char, VOID_STAR, unsigned int,
-				     unsigned char, VOID_STAR)),
+SLclass_add_typecast (SLtype /* from */, SLtype /* to */,
+		      int (*)_PROTO((SLtype, VOID_STAR, unsigned int,
+				     SLtype, VOID_STAR)),
 		      int	       /* allow implicit typecasts */
 		      );
 
-extern char *SLclass_get_datatype_name (unsigned char);
+extern char *SLclass_get_datatype_name (SLtype);
 
 extern double SLcomplex_abs (double *);
 extern double *SLcomplex_times (double *, double *, double *);
@@ -592,16 +605,16 @@ typedef int SLang_MMT_Type;
 
 extern void SLang_free_mmt (SLang_MMT_Type *);
 extern VOID_STAR SLang_object_from_mmt (SLang_MMT_Type *);
-extern SLang_MMT_Type *SLang_create_mmt (unsigned char, VOID_STAR);
+extern SLang_MMT_Type *SLang_create_mmt (SLtype, VOID_STAR);
 extern int SLang_push_mmt (SLang_MMT_Type *);
-extern SLang_MMT_Type *SLang_pop_mmt (unsigned char);
+extern SLang_MMT_Type *SLang_pop_mmt (SLtype);
 extern void SLang_inc_mmt (SLang_MMT_Type *);
 
 /* Maximum number of dimensions of an array. */
 #define SLARRAY_MAX_DIMS		7
 typedef struct _SLang_Array_Type
 {
-   unsigned char data_type;
+   SLtype data_type;
    unsigned int sizeof_type;
    VOID_STAR data;
    unsigned int num_elements;
@@ -619,14 +632,16 @@ typedef struct _SLang_Array_Type
 #define SLARR_DATA_VALUE_IS_INTRINSIC		8
    SLang_Class_Type *cl;
    unsigned int num_refs;
+   void (*free_fun)_PROTO((struct _SLang_Array_Type *));
+   VOID_STAR client_data;
 }
 SLang_Array_Type;
 
-extern int SLang_pop_array_of_type (SLang_Array_Type **, unsigned char);
+extern int SLang_pop_array_of_type (SLang_Array_Type **, SLtype);
 extern int SLang_pop_array (SLang_Array_Type **, int);
 extern int SLang_push_array (SLang_Array_Type *, int);
 extern void SLang_free_array (SLang_Array_Type *);
-extern SLang_Array_Type *SLang_create_array (unsigned char, int, VOID_STAR, int *, unsigned int);
+extern SLang_Array_Type *SLang_create_array (SLtype, int, VOID_STAR, int *, unsigned int);
 extern SLang_Array_Type *SLang_duplicate_array (SLang_Array_Type *);
 extern int SLang_get_array_element (SLang_Array_Type *, int *, VOID_STAR);
 extern int SLang_set_array_element (SLang_Array_Type *, int *, VOID_STAR);
@@ -766,13 +781,16 @@ extern int SLang_init_import (void);
    extern int SLdo_pop_n(unsigned int);
    /* pops n items off stack and frees any memory associated with them */
 
+extern int SLang_pop_datatype (SLtype *);
+extern int SLang_push_datatype (SLtype);
+
 extern int SLang_pop_integer(int *);
 extern int SLang_pop_uinteger(unsigned int *);
    /* pops integer *p0 from the stack.  Returns 0 upon success and non-zero
     * if the stack is empty or a type mismatch occurs, setting SLang_Error.
     */
 extern int SLang_pop_char (char *);
-extern int SLang_pop_uchar (unsigned char *);
+extern int SLang_pop_uchar (SLtype *);
 extern int SLang_pop_short(short *);
 extern int SLang_pop_ushort(unsigned short *);
 extern int SLang_pop_long(long *);
@@ -794,7 +812,7 @@ extern int SLang_pop_double(double *, int *, int *);
    extern int SLang_push_complex (double, double);
 
    extern int SLang_push_char (char);
-   extern int SLang_push_uchar (unsigned char);
+   extern int SLang_push_uchar (SLtype);
 
    extern int SLang_push_integer(int);
    extern int SLang_push_uinteger(unsigned int);
@@ -822,9 +840,9 @@ extern int SLang_pop_double(double *, int *, int *);
 extern int SLang_push_null (void);
 extern int SLang_pop_null (void);
 
-extern int SLang_push_value (unsigned char type, VOID_STAR);
-extern int SLang_pop_value (unsigned char type, VOID_STAR);
-extern void SLang_free_value (unsigned char type, VOID_STAR);
+extern int SLang_push_value (SLtype type, VOID_STAR);
+extern int SLang_pop_value (SLtype type, VOID_STAR);
+extern void SLang_free_value (SLtype type, VOID_STAR);
 
 typedef struct _SLang_Object_Type SLang_Any_Type;
 
@@ -840,7 +858,7 @@ typedef int SLang_Ref_Type;
 
 extern int SLang_pop_ref (SLang_Ref_Type **);
 extern void SLang_free_ref (SLang_Ref_Type *);
-extern int SLang_assign_to_ref (SLang_Ref_Type *, unsigned char, VOID_STAR);
+extern int SLang_assign_to_ref (SLang_Ref_Type *, SLtype, VOID_STAR);
 extern SLang_Name_Type *SLang_pop_function (void);
 extern SLang_Name_Type *SLang_get_fun_from_ref (SLang_Ref_Type *);
 extern void SLang_free_function (SLang_Name_Type *f);
@@ -872,7 +890,7 @@ extern void SLang_doerror(char *);
    /* set SLang_Error and display p1 as error message */
 
 extern int SLang_add_intrinsic_array (char *,   /* name */
-				      unsigned char,   /* type */
+				      SLtype,   /* type */
 				      int,   /* readonly */
 				      VOID_STAR,   /* data */
 				      unsigned int, ...);   /* num dims */
@@ -903,7 +921,7 @@ extern void SLmake_lut (unsigned char *, unsigned char *, unsigned char);
 
 extern int SLstruct_create_struct (unsigned int,
 				   char **,
-				   unsigned char *,
+				   SLtype *,
 				   VOID_STAR *);
 
 /*}}}*/
@@ -978,7 +996,7 @@ extern SLFile_FD_Type *SLfile_dup_fd (SLFile_FD_Type *f0);
 extern int SLang_init_posix_io (void);
 
 typedef double (*SLang_To_Double_Fun_Type)(VOID_STAR);
-extern SLang_To_Double_Fun_Type SLarith_get_to_double_fun (unsigned char, unsigned int *);
+extern SLang_To_Double_Fun_Type SLarith_get_to_double_fun (SLtype, unsigned int *);
 
 extern int SLang_set_argc_argv (int, char **);
 
@@ -1095,7 +1113,7 @@ extern int SLkm_define_key (char *, FVOID_STAR, SLKeyMap_List_Type *);
 
 extern int SLang_define_key(char *, char *, SLKeyMap_List_Type *);
 /* Like define_key1 except that p2 is a string that is to be associated with
- * a function in the functions field of p3.  This routine calls define_key1.
+ * a function in the functions field of p3.
  */
 
 extern int SLkm_define_keysym (char *, unsigned int, SLKeyMap_List_Type *);
@@ -1282,12 +1300,19 @@ extern void SLtt_set_alt_char_set (int);
 extern int SLtt_write_to_status_line (char *, int);
 extern void SLtt_disable_status_line (void);
 # ifdef REAL_UNIX_SYSTEM
-   extern char *SLtt_tgetstr (char *);
-   extern int SLtt_tgetnum (char *);
-   extern int SLtt_tgetflag (char *);
-   extern char *SLtt_tigetent (char *);
-   extern char *SLtt_tigetstr (char *, char **);
-   extern int SLtt_tigetnum (char *, char **);
+/* These are termcap/terminfo routines that assume SLtt_initialize has
+ * been called.
+ */
+extern char *SLtt_tgetstr (char *);
+extern int SLtt_tgetnum (char *);
+extern int SLtt_tgetflag (char *);
+
+/* The following are terminfo-only routines -- these prototypes will change
+ * in V2.x.
+ */
+extern char *SLtt_tigetent (char *);
+extern char *SLtt_tigetstr (char *, char **);
+extern int SLtt_tigetnum (char *, char **);
 # endif
 #endif
 
@@ -1535,8 +1560,15 @@ extern int SLkp_define_keysym (char *, unsigned int);
 /* This function must be called AFTER SLtt_get_terminfo and not before. */
 extern int SLkp_init (void);
 
+/* By default, SLang_getkey is used as the low-level function.  This hook
+ * allows you to specify something else.
+ */
+extern void SLkp_set_getkey_function (int (*)(void));
+
 /* This function uses SLang_getkey and assumes that what ever initialization
- * is required for SLang_getkey has been performed.
+ * is required for SLang_getkey has been performed.  If you do not want 
+ * SLang_getkey to be used, then specify another function via
+ * SLkp_set_getkey_function.
  */
 extern int SLkp_getkey (void);
 
@@ -1632,6 +1664,8 @@ extern int SLerrno_set_errno (int);
 #define SLANG_BSTRING_TYPE	0x25
 #define SLANG_FILE_FD_TYPE	0x26
 
+#define _SLANG_MIN_UNUSED_TYPE	0x27
+
 /* Compatibility */
 #ifdef FLOAT_TYPE
 # undef FLOAT_TYPE
@@ -1692,8 +1726,6 @@ int SLadd_intrinsic_function (char *, FVOID_STAR, unsigned char, unsigned int,..
 int SLns_add_intrinsic_variable (SLang_NameSpace_Type *, char *, VOID_STAR, unsigned char, int);
 int SLns_add_intrinsic_function (SLang_NameSpace_Type *, char *, FVOID_STAR, unsigned char, unsigned int,...);
 
-extern void SLadd_at_handler (long *, char *);
-
 #define MAKE_INTRINSIC_N(n,f,out,in,a1,a2,a3,a4,a5,a6,a7) \
     {(n), NULL, SLANG_INTRINSIC, (FVOID_STAR) (f), \
       {a1,a2,a3,a4,a5,a6,a7}, (in), (out)}
