@@ -1,4 +1,3 @@
-
 /* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
@@ -1122,7 +1121,7 @@ int SLang_pop_double(double *x, int *convertp, int *ip)
 	break;
 
       case SLANG_INT_TYPE:
-	i = (int) obj.v.long_val;
+	i = obj.v.int_val;
 	*x = (double) i;
 	convert = 1;
 	break;
@@ -1278,6 +1277,9 @@ static int integer_to_bool (unsigned char type, int *t)
    return SLang_pop_integer (t);
 }
 
+/* Note that integer literals are all stored in the byte-code as longs.  This
+ * is why it is necessary to use *(long*).
+ */
 static int push_int_literal (unsigned char type, VOID_STAR ptr)
 {
    return SLclass_push_int_obj (type, (int) *(long *) ptr);
