@@ -1143,7 +1143,7 @@ static int push_string_element (unsigned char type, unsigned char *s, unsigned i
 
 /* ARRAY[i, j, k] generates code: __args i j ...k ARRAY __aput/__aget
  * Here i, j, ... k may be a mixture of integers and 1-d arrays, or
- * a single 2-d array of indices.  The 2-d index array is generated by the
+ * a single array of indices.  The index array is generated by the
  * 'where' function.
  *
  * If ARRAY is of type DataType, then this function will create an array of
@@ -1235,13 +1235,32 @@ aput_get_array_to_put (SLang_Class_Type *cl, unsigned int num_elements, int allo
 		       SLang_Array_Type **at_ptr, char **data_to_put, unsigned int *data_increment)
 {
    unsigned char data_type;
+   int type;
    SLang_Array_Type *at;
-
+   
    *at_ptr = NULL;
 
    data_type = cl->cl_data_type;
-   if (-1 == SLclass_typecast (data_type, 1, allow_array))
-     return -1;
+   type = SLang_peek_at_stack ();
+
+   if (type != data_type)
+     {
+	if ((type != SLANG_NULL_TYPE)
+	    || ((cl->cl_class_type != SLANG_CLASS_TYPE_PTR)
+		&& (cl->cl_class_type != SLANG_CLASS_TYPE_MMT)))
+	  {
+	     if (-1 == SLclass_typecast (data_type, 1, allow_array))
+	       return -1;
+	  }
+	else
+	  {
+	     /* This bit of code allows, e.g., a[10] = NULL; */
+	     *data_increment = 0;
+	     *data_to_put = (char *) cl->cl_transfer_buf;
+	     *((char **)cl->cl_transfer_buf) = NULL;
+	     return SLdo_pop ();
+	  }
+     }
 
    if ((data_type != SLANG_ARRAY_TYPE)
        && (data_type != SLANG_ANY_TYPE)
@@ -2042,19 +2061,24 @@ static SLang_Array_Type *concat_arrays (unsigned int count)
 	dest_data += num_elements * sizeof_type;
      }
 
+#if 0   
    /* If the arrays are all 1-d, and all the same size, then reshape to a
     * 2-d array.  This will allow us to do, e.g.
     * a = [[1,2], [3,4]]
     * to specifiy a 2-d.
     * Someday I will generalize this.
     */
+   /* This is a bad idea.  Everyone using it expects concatenation to happen.
+    * Perhaps I will extend the syntax to allow a 2-d array to be expressed
+    * as [[1,2];[3,4]].
+    */
    if ((max_dims == min_dims) && (max_dims == 1) && (min_rows == max_rows))
      {
 	at->num_dims = 2;
 	at->dims[0] = count;
 	at->dims[1] = min_rows;
      }
-
+#endif
    free_and_return:
 
    for (i = 0; i < count; i++)
@@ -2307,6 +2331,43 @@ static int array_binary_op (int op,
    return -1;
 }
 
+static void is_null_intrinsic (SLang_Array_Type *at)
+{
+   SLang_Array_Type *bt;
+
+   bt = SLang_create_array (SLANG_CHAR_TYPE, 0, NULL, at->dims, at->num_dims);
+   if (bt == NULL)
+     return;
+
+   if (at->flags & SLARR_DATA_VALUE_IS_POINTER)
+     {
+	char *cdata, *cdata_max;
+	char **data;
+
+	if (-1 == coerse_array_to_linear (at))
+	  {
+	     SLang_free_array (bt);
+	     return;
+	  }
+	
+	cdata = (char *)bt->data;
+	cdata_max = cdata + bt->num_elements;
+	data = (char **)at->data;
+	
+	while (cdata < cdata_max)
+	  {
+	     if (*data == NULL)
+	       *cdata = 1;
+	     
+	     data++;
+	     cdata++;
+	  }
+     }
+   
+   SLang_push_array (bt, 1);
+}
+   
+   
 static void array_where (void)
 {
    SLang_Array_Type *at, *bt;
@@ -2622,6 +2683,7 @@ static SLang_Intrin_Fun_Type Array_Table [] =
    MAKE_INTRINSIC_1("array_to_bstring", array_to_bstring, SLANG_VOID_TYPE, SLANG_ARRAY_TYPE),
    MAKE_INTRINSIC_1("bstring_to_array", bstring_to_array, SLANG_VOID_TYPE, SLANG_BSTRING_TYPE),
    MAKE_INTRINSIC("init_char_array", init_char_array, SLANG_VOID_TYPE, 0),
+   MAKE_INTRINSIC_1("_isnull", is_null_intrinsic, SLANG_VOID_TYPE, SLANG_ARRAY_TYPE),
    MAKE_INTRINSIC("array_info", array_info, SLANG_VOID_TYPE, 0),
    MAKE_INTRINSIC("where", array_where, SLANG_VOID_TYPE, 0),
    MAKE_INTRINSIC_2("reshape", array_reshape, SLANG_VOID_TYPE, SLANG_ARRAY_TYPE, SLANG_ARRAY_TYPE),
