@@ -545,10 +545,9 @@ int SLclass_register_class (SLang_Class_Type *cl, unsigned char type, unsigned i
 					* for all data types.
 					*/
 
-   if (type == SLANG_VOID_TYPE) for (i = 0; i < 256; i++)
+   if (type == SLANG_VOID_TYPE) for (i = _SLANG_MIN_UNUSED_TYPE; i < 256; i++)
      {
-	if ((Registered_Types[i] == NULL)
-	    && (i != SLANG_VOID_TYPE))
+	if (Registered_Types[i] == NULL)
 	  {
 	     type = (unsigned char) i;
 	     break;
