@@ -136,9 +136,11 @@
 #define HAVE_GETCWD 1
 
 #ifndef VMS
-# define HAVE_VFSCANF 1
+# define HAVE_VFSCANF	1
+# define HAVE_STRTOD	1
 #endif
 
+
 /* Does VMS have this??? */
 #if !defined(VMS) && !defined(_MSC_VER) && (!defined(__WATCOMC__) || defined(__QNX__))
 # define HAVE_DIRENT_H	1
@@ -222,7 +224,7 @@
 #else				       /* NOT VMS */
 
 # define SIZEOF_SHORT	2
-#if defined(__WIN16__) || defined(__MSDOS_16BIT__)
+# if defined(__WIN16__) || defined(__MSDOS_16BIT__)
 # define SIZEOF_INT	2
 #else
 # define SIZEOF_INT	4
