@@ -4,6 +4,7 @@
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
+
 #include "slinclud.h"
 
 #include <time.h>
@@ -880,20 +881,26 @@ void SLtt_beep (void)
    SLtt_flush_output ();
 }
 
+static void write_string_with_care (char *);
+
 static void del_eol (void)
 {
+#if 0
    int c;
-
-   if (Del_Eol_Str != NULL)
+#endif
+   if ((Del_Eol_Str != NULL)
+       && (Can_Background_Color_Erase || ((Current_Fgbg & ~0xFF) == 0)))
      {
 	tt_write_string(Del_Eol_Str);
 	return;
      }
 
+#if 0
    c = Cursor_c;
+   
    /* Avoid writing to the lower right corner.  If the terminal does not
     * have Del_Eol_Str, then it probably does not have what it takes to play
-    * games with insert for for a space into that corner.
+    * games with insert-mode to "push" the desired character into that corner.
     */
    if (Cursor_r + 1 < SLtt_Screen_Rows)
      c++;
@@ -903,6 +910,16 @@ static void del_eol (void)
 	tt_write (" ", 1);
 	c++;
      }
+   Cursor_c = (SLtt_Screen_Cols-1);
+#else
+   while (Cursor_c < SLtt_Screen_Cols)
+     {
+	write_string_with_care (" ");
+	Cursor_c++;
+     }
+   Cursor_c = SLtt_Screen_Cols - 1;
+   Cursor_Set = 0;
+#endif
 }
 
 void SLtt_del_eol (void)
@@ -1388,19 +1405,21 @@ void SLtt_wide_width (void)
 
 /* Highest bit represents the character set. */
 #define COLOR_MASK 0x7F00
+#define COLOR_OF(x) (((x)&COLOR_MASK)>>8)
+#define CHAR_OF(x) ((x)&0x80FF)
 
 #if SLTT_HAS_NON_BCE_SUPPORT
 static int bce_color_eqs (unsigned int a, unsigned int b)
 {
-   a = (a & COLOR_MASK) >> 8;
-   b = (b & COLOR_MASK) >> 8;
+   a = COLOR_OF(a);
+   b = COLOR_OF(b);
    
    if (a == b)
      return 1;
 
    if (SLtt_Use_Ansi_Colors == 0)
      return Ansi_Color_Map[a].mono == Ansi_Color_Map[b].mono;
-   
+
    if (Bce_Color_Offset == 0)
      return Ansi_Color_Map[a].fgbg == Ansi_Color_Map[b].fgbg;
    
@@ -1413,7 +1432,6 @@ static int bce_color_eqs (unsigned int a, unsigned int b)
 }
 #define COLOR_EQS(a,b) bce_color_eqs (a,b)
 #else
-# define COLOR_OF(x) (((unsigned int)(x) & COLOR_MASK) >> 8)
 # define COLOR_EQS(a, b) \
    (SLtt_Use_Ansi_Colors \
     ? (Ansi_Color_Map[COLOR_OF(a)].fgbg == Ansi_Color_Map[COLOR_OF(b)].fgbg)\
@@ -1421,8 +1439,7 @@ static int bce_color_eqs (unsigned int a, unsigned int b)
 #endif
 
 #define CHAR_EQS(a, b) (((a) == (b))\
-			|| ((((a) & ~COLOR_MASK) == ((b) & ~COLOR_MASK))\
-			    && COLOR_EQS((a), (b))))
+			|| ((CHAR_OF(a)==CHAR_OF(b)) && COLOR_EQS(a,b)))
 
 /* The whole point of this routine is to prevent writing to the last column
  * and last row on terminals with automatic margins.
@@ -1529,9 +1546,25 @@ static void forward_cursor (unsigned int n, int row)
 {
    char buf [1024];
 
+
+   /* if (Current_Fgbg & ~0xFF) */
+   /*   { */
+   /* 	unsigned int num = 0; */
+   /* 	while (num < n) */
+   /* 	  { */
+   /* 	     write_string_with_care (" "); */
+   /* 	     num++; */
+   /* 	  } */
+   /* 	Cursor_c += n; */
+   /* 	return; */
+   /*   } */
+   
+	
    if (n <= 4)
      {
 	SLtt_normal_video ();
+	if (n >= sizeof (buf))
+	  n = sizeof (buf) - 1;
 	SLMEMSET (buf, ' ', n);
 	buf[n] = 0;
 	write_string_with_care (buf);
@@ -1547,6 +1580,12 @@ static void forward_cursor (unsigned int n, int row)
 }
 
 
+/* FIXME!!  If the terminal does not support color, then this route has 
+ * problems of color object 0 has been assigned some monochrome attribute
+ * such as reverse video.  In such a case, space_char=' ' is not a simple
+ * space character as is assumed below.
+ */
+
 void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int row)
 {
    register SLsmg_Char_Type *p, *q, *qmax, *pmax, *buf;
@@ -1638,7 +1677,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
    /* Find where the last non-blank character on old/new screen is */
 
    space_char = ' ';
-   if ((*(pmax-1) & 0xFF) == ' ')
+   if (CHAR_EQS(*(pmax-1), ' '))
      {
 	/* If we get here, then we can erase to the end of the line to create
 	 * the final space.  However, this will only work _if_ erasing will 
@@ -1689,7 +1728,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
      {
 #endif
 	/* Try use use erase to bol if possible */
-	if ((Del_Bol_Str != NULL) && ((*neww & 0xFF) == 32))
+	if ((Del_Bol_Str != NULL) && (CHAR_OF(*neww) == ' '))
 	  {
 	     SLsmg_Char_Type *p1;
 	     SLsmg_Char_Type blank;
@@ -1700,7 +1739,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 	       blank = *p1;
 	     /* black+white attributes do not support bce */
 	     else
-	       blank = 32;
+	       blank = ' ';
 
 	     while ((p1 < pmax) && (CHAR_EQS (*p1, blank)))
 	       p1++;
@@ -1718,7 +1757,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 		  q = oldd + ofs;
 		  p = p1;
 		  SLtt_goto_rc (row, ofs - 1);
-		  SLtt_reverse_video (blank >> 8);
+		  SLtt_reverse_video (COLOR_OF(blank));
 		  tt_write_string (Del_Bol_Str);
 		  tt_write (" ", 1);
 		  Cursor_c += 1;
@@ -1740,7 +1779,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 	n_spaces = 0;
 	while (p < pmax)
 	  {
-	     if (CHAR_EQS(*q, 32) && CHAR_EQS(*p, 32))
+	     if (CHAR_EQS(*q, ' ') && CHAR_EQS(*p, ' '))
 	       {
 		  /* If *q is not a space, we would have to overwrite it.
 		   * However, if *q is a space, then while *p is also one,
@@ -1749,8 +1788,8 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 		  space_match = p;
 		  p++; q++;
 		  while ((p < pmax)
-			 && CHAR_EQS(*q, 32)
-			 && CHAR_EQS(*p, 32))
+			 && CHAR_EQS(*q, ' ')
+			 && CHAR_EQS(*p, ' '))
 		    {
 		       p++;
 		       q++;
@@ -1811,20 +1850,22 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 	  }
 	*buf = 0;
 
+	/* At this point, the buffer contains characters that do not match */
 	if (buf != buffer) send_attr_str (buffer);
 	buf = buffer;
 
 	if (n_spaces 
 	    && ((p < pmax) 	       /* erase to eol will achieve this effect*/
-		|| (space_char != 32)))/* unless space_char is not a simple space */
+		|| (space_char != ' ')))/* unless space_char is not a simple space */
 	  {
 	     forward_cursor (n_spaces, row);
 	  }
-
 	/* Now we overwrote what we could and cursor is placed at position
 	 * of a possible match of new and old.  If this is the case, skip
 	 * some more.
 	 */
+	
+	/* Note that from here on, the buffer will contain matched characters */
 #if !SLANG_HAS_KANJI_SUPPORT
 	while ((p < pmax) && CHAR_EQS(*p, *q))
 	  {
@@ -1888,6 +1929,12 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 	  }
      }
 
+   /* At this point we have reached the end of the new string with the 
+    * exception of space_chars hanging off the end of it, but we may not have
+    * reached the end of the old string if they did not match.
+    */
+   
+   /* Here the buffer will consist only of characters that have matched */
    if (buf != buffer)
      {
 	if (q < qmax)
@@ -1907,7 +1954,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 
    if (q < qmax) 
      {
-	SLtt_reverse_video (space_char >> 8);
+	SLtt_reverse_video (COLOR_OF(space_char));
 	del_eol ();
      }
    
@@ -2215,7 +2262,7 @@ int SLtt_initialize (char *term)
    if (is_xterm && (Del_Bol_Str == NULL))
      Del_Bol_Str = "\033[1K";
    if (is_xterm && (Del_Eol_Str == NULL))
-     Del_Bol_Str = "\033[K";
+     Del_Eol_Str = "\033[K";
 
    Rev_Vid_Str = SLtt_tgetstr("mr");
    if (Rev_Vid_Str == NULL) Rev_Vid_Str = SLtt_tgetstr("so");
@@ -2385,6 +2432,12 @@ void SLtt_get_terminfo ()
 {
    int zero = 0;
 
+   /* Apparantly, this cannot fail according to the man pages. */
+   if (SLang_TT_Write_FD == -1)
+     SLang_TT_Write_FD = fileno (stdout);
+   
+   Can_Background_Color_Erase = 0;
+
    Color_Fg_Str = "\033[3%dm";
    Color_Bg_Str = "\033[4%dm";
    Max_Terminfo_Colors = 8;
