@@ -104,10 +104,14 @@ static int import_from_library (char *name,
    char *err;
    char filebuf[1024];
    char *fun_name;
+   char *save_file;
+   char *save_err;
 
    if (NULL != find_handle (name))
      return 0;			       /* already loaded */
-
+   
+   save_err = NULL;
+   save_file = file;
    while (1)
      {
 #ifndef RTLD_GLOBAL
@@ -120,23 +124,36 @@ static int import_from_library (char *name,
 #endif
 
 	if (handle != NULL)
-	  break;
-
-	if (NULL == strchr (file, '/'))
 	  {
-	     _SLsnprintf (filebuf, sizeof (filebuf), "./%s", file);
-	     file = filebuf;
-	     continue;
+	     if (save_err != NULL)
+	       SLfree (save_err);
+	     break;
 	  }
 
 	/* Purify reports that dlerror returns a pointer that generates UMR
 	 * errors.  There is nothing that I can do about that....
 	 */
-	if (NULL == (err = (char *) dlerror ()))
+	if (NULL == strchr (file, '/'))
+	  {
+	     err = (char *) dlerror ();
+	     if (err != NULL)
+	       save_err = SLmake_string (err);
+
+	     _SLsnprintf (filebuf, sizeof (filebuf), "./%s", file);
+	     file = filebuf;
+	     continue;
+	  }
+
+	if ((NULL == (err = save_err))
+	    && (NULL == (err = (char *) dlerror ())))
 	  err = "UNKNOWN";
 
 	SLang_verror (SL_INTRINSIC_ERROR,
-		      "Error linking to %s: %s", file, err);
+		      "Error linking to %s: %s", save_file, err);
+	
+	if (save_err != NULL)
+	  SLfree (save_err);
+
 	return -1;
      }
 
