@@ -47,7 +47,16 @@ SLang_NameSpace_Type *_SLns_allocate_namespace (char *name, unsigned int size)
 {
    SLang_NameSpace_Type *table_list;
    SLang_Name_Type **nt;
-   
+   static int num;
+   char namebuf[64];
+
+   if (name == NULL)
+     {
+	sprintf (namebuf, " *** internal ns <%d> *** ", num);
+	name = namebuf;
+	num++;
+     }
+
    if (NULL != (table_list = find_name_table (name)))
      return table_list;
 
@@ -86,6 +95,9 @@ int _SLns_set_namespace_name (SLang_NameSpace_Type *t, char *name)
    SLang_NameSpace_Type *t1;
    
    t1 = _SLns_find_namespace (name);
+   if (t == t1)
+     return 0;			       /* already has this name */
+
    if (t1 == NULL)
      t1 = t;
    
@@ -96,6 +108,13 @@ int _SLns_set_namespace_name (SLang_NameSpace_Type *t, char *name)
 	return -1;
      }
 
+   if (t->namespace_name != NULL)
+     {
+	SLang_verror (SL_INTRINSIC_ERROR, "An attempt was made to redefine namespace from \"%s\" to \"%s\"\n",
+		      t->namespace_name, name);
+	return -1;
+     }
+
    if (NULL == (name = SLang_create_slstring (name)))
      return -1;
 
@@ -210,8 +229,6 @@ SLang_Array_Type *_SLnspace_apropos (SLang_NameSpace_Type *ns, char *pat, unsign
 SLang_NameSpace_Type *SLns_create_namespace (char *namespace_name)
 {   
    SLang_NameSpace_Type *ns;
-   static int num;
-   char name[64];
 
    if (namespace_name == NULL)
      namespace_name = "Global";
@@ -220,12 +237,9 @@ SLang_NameSpace_Type *SLns_create_namespace (char *namespace_name)
    if (ns != NULL)
      return ns;
    
-   sprintf (name, " *** internal ns <%d> *** ", num);
-
-   if (NULL == (ns = _SLns_allocate_namespace (name, SLSTATIC_HASH_TABLE_SIZE)))
+   if (NULL == (ns = _SLns_allocate_namespace (NULL, SLSTATIC_HASH_TABLE_SIZE)))
      return NULL;
-   
-   num++;
+
    if (-1 == _SLns_set_namespace_name (ns, namespace_name))
      {
 	SLns_delete_namespace (ns);
