@@ -158,7 +158,7 @@ static unsigned char *regexp_looking_at (Re_Context_Type *ctx, register unsigned
 	     n1 = (int) (unsigned char) *buf++;
 	     /* maximum number to match */
 
-	     while (n && (str < estr) && (p1 != *str))
+	     while (n && (str < estr) && (p1 != UPPERCASE(*str)))
 	       {
 		  n--;
 		  str++;
@@ -167,7 +167,7 @@ static unsigned char *regexp_looking_at (Re_Context_Type *ctx, register unsigned
 
 	     save_str = str;
 	     n = n1 - n0;
-	     while (n && (str < estr) && (p1 != *str))
+	     while (n && (str < estr) && (p1 != UPPERCASE(*str)))
 	       {
 		  n--;
 		  str++;
@@ -204,7 +204,7 @@ static unsigned char *regexp_looking_at (Re_Context_Type *ctx, register unsigned
 	     n1 = (int) (unsigned char) *buf++;
 	     /* maximum number to match */
 
-	     while (n && (str < estr) && (p1 == *str))
+	     while (n && (str < estr) && (p1 == UPPERCASE(*str)))
 	       {
 		  n--;
 		  str++;
@@ -213,7 +213,7 @@ static unsigned char *regexp_looking_at (Re_Context_Type *ctx, register unsigned
 
 	     save_str = str;
 	     n = n1 - n0;
-	     while (n && (str < estr) && (p1 == *str))
+	     while (n && (str < estr) && (p1 == UPPERCASE(*str)))
 	       {
 		  n--;
 		  str++;
