@@ -99,7 +99,7 @@ static void blank_line (SLsmg_Char_Type *p, int n, unsigned char ch)
      }
 }
 
-static void clear_region (int row, int n)
+static void clear_region (int row, int n, unsigned char ch)
 {
    int i;
    int imax = row + n;
@@ -109,7 +109,7 @@ static void clear_region (int row, int n)
      {
 	if (i >= 0)
 	  {
-	     blank_line (SL_Screen[i].neew, Screen_Cols, ' ');
+	     blank_line (SL_Screen[i].neew, Screen_Cols, ch);
 	     SL_Screen[i].flags |= TOUCHED;
 	  }
      }
@@ -169,7 +169,7 @@ void SLsmg_erase_eos (void)
    if (Smg_Inited == 0) return;
 
    SLsmg_erase_eol ();
-   clear_region (This_Row + 1, Screen_Rows);
+   clear_region (This_Row + 1, Screen_Rows, ' ');
 }
 
 static int This_Alt_Char;
@@ -485,7 +485,7 @@ void SLsmg_cls (void)
 
    tac = This_Alt_Char; This_Alt_Char = 0;
    SLsmg_set_color (0);
-   clear_region (0, Screen_Rows);
+   clear_region (0, Screen_Rows, ' ');
    This_Alt_Char = tac;
    SLsmg_set_color (0);
    Cls_Flag = 1;
