@@ -1,5 +1,5 @@
 /* file stdio intrinsics for S-Lang */
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -41,7 +41,7 @@
 
 #include <errno.h>
 
-#define SL_APP_WANTS_FOREACH
+/* #define SL_APP_WANTS_FOREACH */
 #include "slang.h"
 #include "_slang.h"
 
@@ -122,7 +122,7 @@ static int open_file_type (char *file, int fd, char *mode,
    SLang_MMT_Type *mmt;
 
    fp = NULL;
-   t = NULL;
+   /* t = NULL; */
    mmt = NULL;
 
    if ((NULL == (t = get_free_file_table_entry ()))
@@ -537,7 +537,7 @@ static void stdio_fread (SLang_Ref_Type *ref, int *data_typep, unsigned int *num
 
    ret = -1;
    s = NULL;
-   cl = NULL;
+   /* cl = NULL; */
 
    if (NULL == (fp = check_fp (t, SL_READ)))
      goto the_return;
@@ -888,7 +888,7 @@ cl_foreach_open (unsigned char type, unsigned int num)
    if (NULL == (mmt = pop_fp (SL_READ, &fp)))
      return NULL;
 
-   type = CTX_USE_LINE;
+   /* type = CTX_USE_LINE; */
 
    switch (num)
      {
