@@ -353,7 +353,7 @@ int _SLcheck_identifier_syntax (char *name)
      }
    
    SLang_verror (SL_SYNTAX_ERROR, 
-		 "Name %s contains an illegal character", name);
+		 "Identifier or structure field name '%s' contains an illegal character", name);
    return -1;
 }
 
@@ -673,7 +673,7 @@ static int get_string_token (_SLang_Token_Type *tok, unsigned char quote_char,
      }
 
    /* else single character */
-   if (s[1] != 0)
+   if ((len == 0) || (s[1] != 0))
      {
 	_SLparse_error("Single char expected", NULL, 0);
 	return (tok->type = EOF_TOKEN);
@@ -928,7 +928,7 @@ int SLang_load_object (SLang_Load_Type *x)
    return 0;
 }
 
-SLang_Load_Type *SLallocate_load_type (char *name)
+SLang_Load_Type *SLns_allocate_load_type (char *name, char *namespace_name)
 {
    SLang_Load_Type *x;
 
@@ -938,20 +938,36 @@ SLang_Load_Type *SLallocate_load_type (char *name)
 
    if (name == NULL) name = "";
 
-   x->name = SLang_create_slstring (name);
-   if (x->name == NULL)
+   if (NULL == (x->name = SLang_create_slstring (name)))
      {
 	SLfree ((char *) x);
 	return NULL;
      }
+   
+   if (namespace_name != NULL)
+     {
+	if (NULL == (x->namespace_name = SLang_create_slstring (namespace_name)))
+	  {
+	     SLang_free_slstring (x->name);
+	     SLfree ((char *) x);
+	     return NULL;
+	  }
+     }
+   
    return x;
 }
 
+SLang_Load_Type *SLallocate_load_type (char *name)
+{
+   return SLns_allocate_load_type (name, NULL);
+}
+
 void SLdeallocate_load_type (SLang_Load_Type *x)
 {
    if (x != NULL)
      {
 	SLang_free_slstring (x->name);
+	SLang_free_slstring (x->namespace_name);
 	SLfree ((char *) x);
      }
 }
@@ -986,6 +1002,11 @@ static char *read_from_string (SLang_Load_Type *x)
 }
 
 int SLang_load_string (char *string)
+{
+   return SLns_load_string (string, NULL);
+}
+
+int SLns_load_string (char *string, char *ns_name)
 {
    SLang_Load_Type *x;
    String_Client_Data_Type data;
@@ -1002,7 +1023,7 @@ int SLang_load_string (char *string)
     * all string objects will be regarded as identical.  So, identify
     * all of them by ***string***
     */
-   if (NULL == (x = SLallocate_load_type ("***string***")))
+   if (NULL == (x = SLns_allocate_load_type ("***string***", ns_name)))
      {
 	SLang_free_slstring (string);
 	return -1;
@@ -1049,23 +1070,32 @@ static char *read_from_file (SLang_Load_Type *x)
  * so get it and store it !! (e.g., autoloading)
  */
 int (*SLang_Load_File_Hook) (char *);
+int (*SLns_Load_File_Hook) (char *, char *);
 int SLang_load_file (char *f)
+{
+   return SLns_load_file (f, NULL);
+}
+
+int SLns_load_file (char *f, char *ns_name)
 {
    File_Client_Data_Type client_data;
    SLang_Load_Type *x;
    char *name, *buf;
    FILE *fp;
 
-   if (SLang_Load_File_Hook != NULL)
+   if ((ns_name == NULL) && (NULL != SLang_Load_File_Hook))
      return (*SLang_Load_File_Hook) (f);
 
+   if (SLns_Load_File_Hook != NULL)
+     return (*SLns_Load_File_Hook) (f, ns_name);
+
    if (f == NULL) name = "<stdin>"; else name = f;
 
    name = SLang_create_slstring (name);
    if (name == NULL)
      return -1;
 
-   if (NULL == (x = SLallocate_load_type (name)))
+   if (NULL == (x = SLns_allocate_load_type (name, ns_name)))
      {
 	SLang_free_slstring (name);
 	return -1;
@@ -1103,230 +1133,6 @@ int SLang_load_file (char *f)
    return 0;
 }
 
-int SLang_guess_type (char *t)
-{
-   char *p;
-   register char ch;
-   int modifier;
-
-   if (*t == '-') t++;
-   p = t;
-
-#if SLANG_HAS_FLOAT
-   if (*p != '.')
-     {
-#endif
-	modifier = 0;
-	while ((*p >= '0') && (*p <= '9')) p++;
-	if (t == p) return (SLANG_STRING_TYPE);
-	if ((*p == 'x') && (p == t + 1))   /* 0x?? */
-	  {
-	     modifier |= 8;
-	     p++;
-	     while (ch = *p,
-		    ((ch >= '0') && (ch <= '9'))
-		    || (((ch | 0x20) >= 'a') && ((ch | 0x20) <= 'f'))) p++;
-	  }
-
-	/* Now look for UL, LU, UH, HU, L, H modifiers */
-	while ((ch = *p) != 0)
-	  {
-	     ch |= 0x20;
-	     if (ch == 'h') modifier |= 1;
-	     else if (ch == 'l') modifier |= 2;
-	     else if (ch == 'u') modifier |= 4;
-	     else break;
-	     p++;
-	  }
-	if ((1|2) == (modifier & (1|2)))	       /* hl present */
-	  return SLANG_STRING_TYPE;
-
-	if (ch == 0)
-	  {
-	     if ((modifier & 0x7) == 0) return SLANG_INT_TYPE;
-	     if (modifier & 4)
-	       {
-		  if (modifier & 1) return SLANG_USHORT_TYPE;
-		  if (modifier & 2) return SLANG_ULONG_TYPE;
-		  return SLANG_UINT_TYPE;
-	       }
-	     if (modifier & 1) return SLANG_SHORT_TYPE;
-	     if (modifier & 2) return SLANG_LONG_TYPE;
-	     return SLANG_INT_TYPE;
-	  }
-
-	if (modifier) return SLANG_STRING_TYPE;
-#if SLANG_HAS_FLOAT
-     }
-
-   /* now down to double case */
-   if (*p == '.')
-     {
-	p++;
-	while ((*p >= '0') && (*p <= '9')) p++;
-     }
-   if (*p == 0) return(SLANG_DOUBLE_TYPE);
-   if ((*p != 'e') && (*p != 'E'))
-     {
-# if SLANG_HAS_COMPLEX
-	if (((*p == 'i') || (*p == 'j'))
-	    && (p[1] == 0))
-	  return SLANG_COMPLEX_TYPE;
-# endif
-	if (((*p | 0x20) == 'f') && (p[1] == 0))
-	  return SLANG_FLOAT_TYPE;
-
-	return SLANG_STRING_TYPE;
-     }
-
-   p++;
-   if ((*p == '-') || (*p == '+')) p++;
-   while ((*p >= '0') && (*p <= '9')) p++;
-   if (*p != 0)
-     {
-# if SLANG_HAS_COMPLEX
-	if (((*p == 'i') || (*p == 'j'))
-	    && (p[1] == 0))
-	  return SLANG_COMPLEX_TYPE;
-# endif
-	if (((*p | 0x20) == 'f') && (p[1] == 0))
-	  return SLANG_FLOAT_TYPE;
-
-	return SLANG_STRING_TYPE;
-     }
-   return SLANG_DOUBLE_TYPE;
-#else
-   return SLANG_STRING_TYPE;
-#endif				       /* SLANG_HAS_FLOAT */
-}
-
-static int hex_atoul (unsigned char *s, unsigned long *ul)
-{
-   register unsigned char ch;
-   register unsigned long value;
-   register int base;
-
-   s++;				       /* skip the leading 0 */
-
-   /* look for 'x' which indicates hex */
-   if ((*s | 0x20) == 'x')
-     {
-	base = 16;
-	s++;
-	if (*s == 0)
-	  {
-	     SLang_Error = SL_SYNTAX_ERROR;
-	     return -1;
-	  }
-     }
-   else base = 8;
-
-   value = 0;
-   while ((ch = *s++) != 0)
-     {
-	char ch1 = ch | 0x20;
-	switch (ch1)
-	  {
-	   default:
-	     SLang_Error = SL_SYNTAX_ERROR;
-	     break;
-
-	   case 'u':
-	   case 'l':
-	   case 'h':
-	     *ul = value;
-	     return 0;
-
-	   case '8':
-	   case '9':
-	     if (base != 16) SLang_Error = SL_SYNTAX_ERROR;
-	     /* drop */
-	   case '0':
-	   case '1':
-	   case '2':
-	   case '3':
-	   case '4':
-	   case '5':
-	   case '6':
-	   case '7':
-	     ch1 -= '0';
-	     break;
-
-	   case 'a':
-	   case 'b':
-	   case 'c':
-	   case 'd':
-	   case 'e':
-	   case 'f':
-	     if (base != 16) SLang_Error = SL_SYNTAX_ERROR;
-	     ch1 = (ch1 - 'a') + 10;
-	     break;
-	  }
-	value = value * base + ch1;
-     }
-   *ul = value;
-   return 0;
-}
-
-/* Note: These routines do not check integer overflow.  I would use the C
- * library functions atol and atoul but some implementations check overflow
- * and some do not.  The following implementations provide a consistent
- * behavior.
- */
-unsigned long SLatoul (unsigned char *s)
-{
-   int sign;
-   unsigned long value;
-
-   if (*s == '-') sign = -1;
-   else
-     {
-	sign = 1;
-	if (*s == '+') s++;
-     }
-
-   if (*s == '0')
-     {
-	if (-1 == hex_atoul (s, &value))
-	  return (unsigned long) -1;
-     }
-   else
-     {
-	while (WHITE_CHAR == CHAR_CLASS(*s))
-	  s++;
-
-	value = 0;
-	while (DIGIT_CHAR == CHAR_CLASS(*s))
-	  {
-	     value = value * 10 + (unsigned long) (*s - '0');
-	     s++;
-	  }
-     }
-
-   if (sign == -1)
-     value = (unsigned long)-1L * value;
-
-   return value;
-}
-
-long SLatol (unsigned char *s)
-{
-   while (WHITE_CHAR == CHAR_CLASS(*s))
-     s++;
-
-   if (*s == '-')
-     {
-	long value = (long) SLatoul (s+1);
-	return -value;
-     }
-   return (long) SLatoul (s);
-}
-
-int SLatoi (unsigned char *s)
-{
-   return (int) SLatol (s);
-}
-
 static char *check_byte_compiled_token (char *buf)
 {
    unsigned int len_lo, len_hi, len;
