@@ -185,15 +185,17 @@ string_string_bin_op (int op,
 
    a = (char **) ap;
    b = (char **) bp;
-   for (n = 0; n < n_max; n++)
-     {
-	if ((*a == NULL) || (*b == NULL))
-	  {
-	     SLang_verror (SL_VARIABLE_UNINITIALIZED, "String element[%u] not initialized for binary operation", n);
-	     return -1;
-	  }
-	a += da; b += db;
-     }
+   
+   if ((op != SLANG_NE) && (op != SLANG_EQ))
+     for (n = 0; n < n_max; n++)
+       {
+	  if ((*a == NULL) || (*b == NULL))
+	    {
+	       SLang_verror (SL_VARIABLE_UNINITIALIZED, "String element[%u] not initialized for binary operation", n);
+	       return -1;
+	    }
+	  a += da; b += db;
+       }
 
    a = (char **) ap;
    b = (char **) bp;
@@ -222,7 +224,11 @@ string_string_bin_op (int op,
       case SLANG_NE:
 	for (n = 0; n < n_max; n++)
 	  {
-	     ic [n] = (0 != strcmp (*a, *b));
+	     if ((*a == NULL) || (*b == NULL))
+	       ic [n] = (*a != *b);
+	     else
+	       ic [n] = (*a != *b) && (0 != strcmp (*a, *b));
+
 	     a += da;
 	     b += db;
 	  }
@@ -262,7 +268,10 @@ string_string_bin_op (int op,
       case SLANG_EQ:
 	for (n = 0; n < n_max; n++)
 	  {
-	     ic [n] = (strcmp (*a, *b) == 0);
+	     if ((*a == NULL) || (*b == NULL))
+	       ic[n] = (*a == *b);
+	     else
+	       ic [n] = (*a == *b) || (strcmp (*a, *b) == 0);
 	     a += da;
 	     b += db;
 	  }
